@@ -7,6 +7,7 @@ from urllib.parse import urlparse, urlencode, quote
 
 from toot import http, CLIENT_NAME, CLIENT_WEBSITE
 from toot.exceptions import AuthenticationError
+from toot.utils import str_bool
 
 SCOPES = 'read write follow'
 
@@ -17,8 +18,14 @@ def _account_action(app, user, account, action):
     return http.post(app, user, url).json()
 
 
-def create_app(domain):
-    url = 'https://{}/api/v1/apps'.format(domain)
+def _status_action(app, user, status_id, action):
+    url = '/api/v1/statuses/{}/{}'.format(status_id, action)
+
+    return http.post(app, user, url).json()
+
+
+def create_app(domain, scheme='https'):
+    url = '{}://{}/api/v1/apps'.format(scheme, domain)
 
     data = {
         'client_name': CLIENT_NAME,
@@ -48,7 +55,7 @@ def login(app, username, password):
     if response.is_redirect:
         raise AuthenticationError()
 
-    return http.process_response(response).json()
+    return response.json()
 
 
 def get_browser_login_url(app):
@@ -72,9 +79,7 @@ def request_access_token(app, authorization_code):
         'redirect_uri': 'urn:ietf:wg:oauth:2.0:oob',
     }
 
-    response = http.anon_post(url, data, allow_redirects=False)
-
-    return http.process_response(response).json()
+    return http.anon_post(url, data, allow_redirects=False).json()
 
 
 def post_status(
@@ -85,7 +90,8 @@ def post_status(
     media_ids=None,
     sensitive=False,
     spoiler_text=None,
-    in_reply_to_id=None
+    in_reply_to_id=None,
+    language=None,
 ):
     """
     Posts a new status.
@@ -100,9 +106,10 @@ def post_status(
         'status': status,
         'media_ids[]': media_ids,
         'visibility': visibility,
-        'sensitive': "true" if sensitive else "false",
+        'sensitive': str_bool(sensitive),
         'spoiler_text': spoiler_text,
         'in_reply_to_id': in_reply_to_id,
+        'language': language,
     }, headers=headers).json()
 
 
@@ -114,27 +121,43 @@ def delete_status(app, user, status_id):
     return http.delete(app, user, '/api/v1/statuses/{}'.format(status_id))
 
 
-def timeline_home(app, user):
-    return http.get(app, user, '/api/v1/timelines/home').json()
+def favourite(app, user, status_id):
+    return _status_action(app, user, status_id, 'favourite')
 
 
-def timeline_public(app, user, local=False):
-    params = {'local': 'true' if local else 'false'}
-    return http.get(app, user, '/api/v1/timelines/public', params).json()
+def unfavourite(app, user, status_id):
+    return _status_action(app, user, status_id, 'unfavourite')
 
 
-def timeline_tag(app, user, hashtag, local=False):
-    url = '/api/v1/timelines/tag/{}'.format(quote(hashtag))
-    params = {'local': 'true' if local else 'false'}
-    return http.get(app, user, url, params).json()
+def reblog(app, user, status_id):
+    return _status_action(app, user, status_id, 'reblog')
 
 
-def timeline_list(app, user, list_id):
-    url = '/api/v1/timelines/list/{}'.format(list_id)
+def unreblog(app, user, status_id):
+    return _status_action(app, user, status_id, 'unreblog')
+
+
+def pin(app, user, status_id):
+    return _status_action(app, user, status_id, 'pin')
+
+
+def unpin(app, user, status_id):
+    return _status_action(app, user, status_id, 'unpin')
+
+
+def context(app, user, status_id):
+    url = '/api/v1/statuses/{}/context'.format(status_id)
+
     return http.get(app, user, url).json()
 
 
-def get_next_path(headers):
+def reblogged_by(app, user, status_id):
+    url = '/api/v1/statuses/{}/reblogged_by'.format(status_id)
+
+    return http.get(app, user, url).json()
+
+
+def _get_next_path(headers):
     """Given timeline response headers, returns the path to the next batch"""
     links = headers.get('Link', '')
     matches = re.match('<([^>]+)>; rel="next"', links)
@@ -143,19 +166,19 @@ def get_next_path(headers):
         return "?".join([parsed.path, parsed.query])
 
 
-def _timeline_generator(app, user, path, limit=20):
+def _timeline_generator(app, user, path, params=None):
     while path:
-        response = http.get(app, user, path)
+        response = http.get(app, user, path, params)
         yield response.json()
-        path = get_next_path(response.headers)
+        path = _get_next_path(response.headers)
 
 
-def _anon_timeline_generator(instance, path, limit=20):
+def _anon_timeline_generator(instance, path, params=None):
     while path:
         url = "https://{}{}".format(instance, path)
-        response = http.anon_get(url, path)
+        response = http.anon_get(url, params)
         yield response.json()
-        path = get_next_path(response.headers)
+        path = _get_next_path(response.headers)
 
 
 def home_timeline_generator(app, user, limit=20):
@@ -163,9 +186,21 @@ def home_timeline_generator(app, user, limit=20):
     return _timeline_generator(app, user, path)
 
 
-def public_timeline_generator(instance, limit=20):
-    path = '/api/v1/timelines/public?limit={}'.format(limit)
-    return _anon_timeline_generator(instance, path)
+def public_timeline_generator(instance, local=False, limit=20):
+    path = '/api/v1/timelines/public'
+    params = {'local': str_bool(local), 'limit': limit}
+    return _anon_timeline_generator(instance, path, params)
+
+
+def tag_timeline_generator(instance, hashtag, local=False, limit=20):
+    path = '/api/v1/timelines/tag/{}'.format(hashtag)
+    params = {'local': str_bool(local), 'limit': limit}
+    return _anon_timeline_generator(instance, path, params)
+
+
+def timeline_list_generator(app, user, list_id, limit=20):
+    path = '/api/v1/timelines/list/{}'.format(list_id)
+    return _timeline_generator(app, user, path, {'limit': limit})
 
 
 def upload_media(app, user, file):
@@ -215,10 +250,20 @@ def verify_credentials(app, user):
     return http.get(app, user, '/api/v1/accounts/verify_credentials').json()
 
 
+def single_status(app, user, status_id):
+    url = '/api/v1/statuses/{}'.format(status_id)
+
+    return http.get(app, user, url).json()
+
+
 def get_notifications(app, user):
     return http.get(app, user, '/api/v1/notifications').json()
 
 
-def get_instance(domain):
-    url = "http://{}/api/v1/instance".format(domain)
+def clear_notifications(app, user):
+    http.post(app, user, '/api/v1/notifications/clear')
+
+
+def get_instance(domain, scheme="https"):
+    url = "{}://{}/api/v1/instance".format(scheme, domain)
     return http.anon_get(url).json()
