@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
 
-import os
-import sys
 import logging
+import os
+import shutil
+import sys
 
 from argparse import ArgumentParser, FileType, ArgumentTypeError
 from collections import namedtuple
@@ -13,8 +14,19 @@ from toot.output import print_out, print_err
 VISIBILITY_CHOICES = ['public', 'unlisted', 'private', 'direct']
 
 
+def language(value):
+    """Validates the language parameter"""
+    if len(value) != 3:
+        raise ArgumentTypeError(
+            "Invalid language specified: '{}'. Expected a 3 letter "
+            "abbreviation according to ISO 639-2 standard.".format(value)
+        )
+
+    return value
+
+
 def visibility(value):
-    """Validates the visibilty parameter"""
+    """Validates the visibility parameter"""
     if value not in VISIBILITY_CHOICES:
         raise ValueError("Invalid visibility value")
 
@@ -28,6 +40,21 @@ def timeline_count(value):
     return n
 
 
+def editor(value):
+    if not value:
+        raise ArgumentTypeError(
+            "Editor not specified in --editor option and $EDITOR environment "
+            "variable not set."
+        )
+
+    # Check editor executable exists
+    exe = shutil.which(value)
+    if not exe:
+        raise ArgumentTypeError("Editor `{}` not found".format(value))
+
+    return exe
+
+
 Command = namedtuple("Command", ["name", "description", "require_auth", "arguments"])
 
 
@@ -85,7 +112,7 @@ status_id_arg = (["status_id"], {
 })
 
 # Arguments for selecting a timeline (see `toot.commands.get_timeline_generator`)
-timeline_args = [
+common_timeline_args = [
     (["-p", "--public"], {
         "action": "store_true",
         "default": False,
@@ -108,13 +135,33 @@ timeline_args = [
         "type": int,
         "help": "show timeline for given list.",
     }),
+]
+
+timeline_args = common_timeline_args + [
     (["-c", "--count"], {
         "type": timeline_count,
         "help": "number of toots to show per page (1-20, default 10).",
         "default": 10,
     }),
+    (["-r", "--reverse"], {
+        "action": "store_true",
+        "default": False,
+        "help": "Reverse the order of the shown timeline (to new posts at the bottom)",
+    }),
+    (["-1", "--once"], {
+        "action": "store_true",
+        "default": False,
+        "help": "Only show the first <count> toots, do not prompt to continue.",
+    }),
 ]
 
+curses_args = common_timeline_args + [
+    (["-c", "--count"], {
+        "type": timeline_count,
+        "help": "number of toots to show per page (1-20, default 20).",
+        "default": 20,
+    }),
+]
 
 AUTH_COMMANDS = [
     Command(
@@ -149,6 +196,21 @@ AUTH_COMMANDS = [
     ),
 ]
 
+TUI_COMMANDS = [
+    Command(
+        name="tui",
+        description="Launches the toot terminal user interface",
+        arguments=[],
+        require_auth=False,
+    ),
+    Command(
+        name="curses",
+        description="An experimental timeline app (DEPRECATED, use 'toot tui' instead)",
+        arguments=curses_args,
+        require_auth=False,
+    ),
+]
+
 
 READ_COMMANDS = [
     Command(
@@ -167,6 +229,18 @@ READ_COMMANDS = [
         ],
         require_auth=True,
     ),
+    Command(
+        name="notifications",
+        description="Notifications for logged in user",
+        arguments=[
+            (["--clear"], {
+                "help": "delete all notifications from the server",
+                "action": 'store_true',
+                "default": False,
+            }),
+        ],
+        require_auth=True,
+    ),
     Command(
         name="instance",
         description="Display instance details",
@@ -207,25 +281,8 @@ READ_COMMANDS = [
     Command(
         name="timeline",
         description="Show recent items in a timeline (home by default)",
-        arguments=timeline_args + [
-            (["-r", "--reverse"], {
-                "action": "store_true",
-                "default": False,
-                "help": "Reverse the order of the shown timeline (to new posts at the bottom)",
-            }),
-            (["-1", "--once"], {
-                "action": "store_true",
-                "default": False,
-                "help": "Only show the first <count> toots, do not prompt to continue.",
-            }),
-        ],
-        require_auth=True,
-    ),
-    Command(
-        name="curses",
-        description="An experimental timeline app (doesn't work on Windows)",
         arguments=timeline_args,
-        require_auth=False,
+        require_auth=True,
     ),
 ]
 
@@ -239,8 +296,10 @@ POST_COMMANDS = [
                 "nargs": "?",
             }),
             (["-m", "--media"], {
-                "type": FileType('rb'),
-                "help": "path to the media file to attach"
+                "action": "append",
+                "type": FileType("rb"),
+                "help": "path to the media file to attach (specify multiple "
+                        "times to attach up to 4 files)"
             }),
             (["-v", "--visibility"], {
                 "type": visibility,
@@ -254,11 +313,22 @@ POST_COMMANDS = [
             }),
             (["-p", "--spoiler-text"], {
                 "type": str,
-                "help": 'text to be shown as a warning before the actual content',
+                "help": "text to be shown as a warning before the actual content",
             }),
             (["-r", "--reply-to"], {
                 "type": int,
-                "help": 'local ID of the status you want to reply to',
+                "help": "local ID of the status you want to reply to",
+            }),
+            (["-l", "--language"], {
+                "type": language,
+                "help": "ISO 639-2 language code of the toot, to skip automatic detection",
+            }),
+            (["-e", "--editor"], {
+                "type": editor,
+                "nargs": "?",
+                "const": os.getenv("EDITOR", ""),  # option given without value
+                "help": "Specify an editor to compose your toot, "
+                        "defaults to editor defined in $EDITOR env variable.",
             }),
         ],
         require_auth=True,
@@ -378,7 +448,7 @@ ACCOUNTS_COMMANDS = [
     ),
 ]
 
-COMMANDS = AUTH_COMMANDS + READ_COMMANDS + POST_COMMANDS + STATUS_COMMANDS + ACCOUNTS_COMMANDS
+COMMANDS = AUTH_COMMANDS + READ_COMMANDS + TUI_COMMANDS + POST_COMMANDS + STATUS_COMMANDS + ACCOUNTS_COMMANDS
 
 
 def print_usage():
@@ -386,6 +456,7 @@ def print_usage():
 
     groups = [
         ("Authentication", AUTH_COMMANDS),
+        ("TUI", TUI_COMMANDS),
         ("Read", READ_COMMANDS),
         ("Post", POST_COMMANDS),
         ("Status", STATUS_COMMANDS),
@@ -462,12 +533,6 @@ def main():
         filename = os.getenv("TOOT_LOG_FILE")
         logging.basicConfig(level=logging.DEBUG, filename=filename)
 
-    # If something is piped in, append it to commandline arguments
-    if not sys.stdin.isatty():
-        stdin = sys.stdin.read()
-        if stdin:
-            sys.argv.append(stdin)
-
     command_name = sys.argv[1] if len(sys.argv) > 1 else None
     args = sys.argv[2:]
 
@@ -481,5 +546,5 @@ def main():
     except (ConsoleError, ApiError) as e:
         print_err(str(e))
         sys.exit(1)
-    except KeyboardInterrupt as e:
+    except KeyboardInterrupt:
         pass
